package lib.ui;

import util.Pos;
import util.Rect;
import util.Vec.Vec2;
import kernel.ui.Pixel;

abstract Canvas(Array<Array<Pixel>>) to Array<Array<Pixel>> {
	inline public function new() {
		this = [[]];
	}

	public inline function set(i:Pos, pixel:Pixel) {
		if (this[i.y] == null) {
			this[i.y] = [];
		}

		this[i.y][i.x] = pixel;
	}

	public inline function get(i:Pos):Pixel {
		return this[i.y][i.x];
	}

	public function keyValueIterator():KeyValueIterator<Pos, Pixel> {
		return new CanvasKeyValueIterator(this);
	}

	public function combine(other:Canvas, offset:Pos) {
		for (key => value in other) {
			if (value == null) {
				continue;
			}

			var y = offset.y + key.y;
			var x = offset.x + key.x;

			if (this[y] == null) {
				this[y] = [];
			}

			this[y][x] = value;
		}
	}

	public function hight():Int {
		return this.length;
	}

	public function maxWidth() {
		var max = 0;
		for (i in 0...this.length) {
			if (this[i].length > max) {
				max = this[i].length;
			}
		}

		return max;
	}

	public function getBounds(): Rect {
		return new Rect({x:0,y:0},{
			x: maxWidth(),
			y: hight()
		});
	}
}

class CanvasKeyValueIterator {
	private final canvas:Array<Array<Pixel>>;
	private var index:Pos = {x: 0, y: 0};

	@:allow(lib.ui.Canvas)
	private function new(canvas:Array<Array<Pixel>>) {
		this.canvas = canvas;
	}

	public function hasNext():Bool {
		return index.y < canvas.length && index.x <= canvas[index.y].length;
	}

	public function next():{key:Pos, value:Pixel} {
		var oldIndex:Pos = this.index;

		if (index.x >= canvas[index.y].length) {
			// Goto next line
			index = {x: 0, y: index.y + 1};
		} else {
			// Goto next pixel in line
			index = {x: index.x + 1, y: index.y};
		}

		return {
			key: oldIndex,
			value: this.canvas[oldIndex.y][oldIndex.x]
		};
	}
}
